@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 // Package backend provides interfaces that the CLI uses to interact with
@@ -22,6 +24,7 @@ import (
 	"github.com/opentofu/opentofu/internal/configs/configload"
 	"github.com/opentofu/opentofu/internal/configs/configschema"
 	"github.com/opentofu/opentofu/internal/depsfile"
+	"github.com/opentofu/opentofu/internal/encryption"
 	"github.com/opentofu/opentofu/internal/plans"
 	"github.com/opentofu/opentofu/internal/plans/planfile"
 	"github.com/opentofu/opentofu/internal/states"
@@ -52,7 +55,7 @@ var (
 )
 
 // InitFn is used to initialize a new backend.
-type InitFn func() Backend
+type InitFn func(encryption.StateEncryption) Backend
 
 // Backend is the minimal interface that must be implemented to enable OpenTofu.
 type Backend interface {
@@ -235,6 +238,9 @@ type Operation struct {
 	// Type is the operation to perform.
 	Type OperationType
 
+	// Encryption is used by enhanced backends for planning and tofu.Context initialization
+	Encryption encryption.Encryption
+
 	// PlanId is an opaque value that backends can use to execute a specific
 	// plan for an apply operation.
 	//
@@ -277,7 +283,9 @@ type Operation struct {
 	AutoApprove  bool
 	Targets      []addrs.Targetable
 	ForceReplace []addrs.AbsResourceInstance
-	Variables    map[string]UnparsedVariableValue
+	// Injected by the command creating the operation (plan/apply/refresh/etc...)
+	Variables map[string]UnparsedVariableValue
+	RootCall  configs.StaticModuleCall
 
 	// Some operations use root module variables only opportunistically or
 	// don't need them at all. If this flag is set, the backend must treat
@@ -320,15 +328,6 @@ func (o *Operation) HasConfig() bool {
 	return o.ConfigLoader.IsConfigDir(o.ConfigDir)
 }
 
-// Config loads the configuration that the operation applies to, using the
-// ConfigDir and ConfigLoader fields within the receiving operation.
-func (o *Operation) Config() (*configs.Config, tfdiags.Diagnostics) {
-	var diags tfdiags.Diagnostics
-	config, hclDiags := o.ConfigLoader.LoadConfig(o.ConfigDir)
-	diags = diags.Append(hclDiags)
-	return config, diags
-}
-
 // ReportResult is a helper for the common chore of setting the status of
 // a running operation and showing any diagnostics produced during that
 // operation.
@@ -413,7 +412,7 @@ func (r OperationResult) ExitStatus() int {
 	return int(r)
 }
 
-// If the argument is a path, Read loads it and returns the contents,
+// If the argument is a path, ReadPathOrContents loads it and returns the contents,
 // otherwise the argument is assumed to be the desired contents and is simply
 // returned.
 func ReadPathOrContents(poc string) (string, error) {
