@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package init
@@ -7,13 +9,14 @@ import (
 	"testing"
 
 	"github.com/opentofu/opentofu/internal/backend/remote-state/inmem"
+	"github.com/opentofu/opentofu/internal/encryption"
 	"github.com/zclconf/go-cty/cty"
 )
 
 func TestDeprecateBackend(t *testing.T) {
 	deprecateMessage := "deprecated backend"
 	deprecatedBackend := deprecateBackend(
-		inmem.New(),
+		inmem.New(encryption.StateEncryptionDisabled()),
 		deprecateMessage,
 	)
 
