@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package remote
@@ -221,7 +223,7 @@ func (b *Remote) waitForRun(stopCtx, cancelCtx context.Context, op *backend.Oper
 // remote system's responsibility to do final validation of the input.
 func (b *Remote) hasExplicitVariableValues(op *backend.Operation) bool {
 	// Load the configuration using the caller-provided configuration loader.
-	config, _, configDiags := op.ConfigLoader.LoadConfigWithSnapshot(op.ConfigDir)
+	config, _, configDiags := op.ConfigLoader.LoadConfigWithSnapshot(op.ConfigDir, op.RootCall)
 	if configDiags.HasErrors() {
 		// If we can't load the configuration then we'll assume no explicit
 		// variable values just to let the remote operation start and let
@@ -465,8 +467,10 @@ func (b *Remote) confirm(stopCtx context.Context, op *backend.Operation, opts *t
 	doneCtx, cancel := context.WithCancel(stopCtx)
 	result := make(chan error, 2)
 
+	panicHandler := logging.PanicHandlerWithTraceFn()
+
 	go func() {
-		defer logging.PanicHandler()
+		defer panicHandler()
 
 		// Make sure we cancel doneCtx before we return
 		// so the input command is also canceled.
