@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package cloud
@@ -7,13 +9,13 @@ import (
 	"context"
 	"fmt"
 	"net/http"
-	"os"
 	"strings"
 	"testing"
 
 	tfe "github.com/hashicorp/go-tfe"
 	version "github.com/hashicorp/go-version"
 	"github.com/opentofu/opentofu/internal/backend"
+	"github.com/opentofu/opentofu/internal/encryption"
 	"github.com/opentofu/opentofu/internal/tfdiags"
 	tfversion "github.com/opentofu/opentofu/version"
 	"github.com/zclconf/go-cty/cty"
@@ -22,8 +24,8 @@ import (
 )
 
 func TestCloud(t *testing.T) {
-	var _ backend.Enhanced = New(nil)
-	var _ backend.CLI = New(nil)
+	var _ backend.Enhanced = New(nil, encryption.StateEncryptionDisabled())
+	var _ backend.CLI = New(nil, encryption.StateEncryptionDisabled())
 }
 
 func TestCloud_backendWithName(t *testing.T) {
@@ -54,7 +56,7 @@ func TestCloud_backendWithName(t *testing.T) {
 
 func TestCloud_backendWithoutHost(t *testing.T) {
 	s := testServer(t)
-	b := New(testDisco(s))
+	b := New(testDisco(s), encryption.StateEncryptionDisabled())
 
 	obj := cty.ObjectVal(map[string]cty.Value{
 		"hostname":     cty.NullVal(cty.String),
@@ -174,7 +176,7 @@ func TestCloud_PrepareConfig(t *testing.T) {
 	for name, tc := range cases {
 		t.Run(name, func(t *testing.T) {
 			s := testServer(t)
-			b := New(testDisco(s))
+			b := New(testDisco(s), encryption.StateEncryptionDisabled())
 
 			// Validate
 			_, valDiags := b.PrepareConfig(tc.config)
@@ -325,16 +327,11 @@ func TestCloud_PrepareConfigWithEnvVars(t *testing.T) {
 	for name, tc := range cases {
 		t.Run(name, func(t *testing.T) {
 			s := testServer(t)
-			b := New(testDisco(s))
+			b := New(testDisco(s), encryption.StateEncryptionDisabled())
 
 			for k, v := range tc.vars {
-				os.Setenv(k, v)
+				t.Setenv(k, v)
 			}
-			t.Cleanup(func() {
-				for k := range tc.vars {
-					os.Unsetenv(k)
-				}
-			})
 
 			_, valDiags := b.PrepareConfig(tc.config)
 			if (valDiags.Err() == nil) != (tc.expectedErr == "") {
@@ -491,7 +488,7 @@ func TestCloud_configVerifyMinimumTFEVersion(t *testing.T) {
 	}
 	s := testServerWithHandlers(handlers)
 
-	b := New(testDisco(s))
+	b := New(testDisco(s), encryption.StateEncryptionDisabled())
 
 	confDiags := b.Configure(config)
 	if confDiags.Err() == nil {
@@ -528,7 +525,7 @@ func TestCloud_configVerifyMinimumTFEVersionInAutomation(t *testing.T) {
 	}
 	s := testServerWithHandlers(handlers)
 
-	b := New(testDisco(s))
+	b := New(testDisco(s), encryption.StateEncryptionDisabled())
 	b.runningInAutomation = true
 
 	confDiags := b.Configure(config)
@@ -1290,7 +1287,7 @@ func TestCloudBackend_DeleteWorkspace_DoesNotExist(t *testing.T) {
 
 func TestCloud_ServiceDiscoveryAliases(t *testing.T) {
 	s := testServer(t)
-	b := New(testDisco(s))
+	b := New(testDisco(s), encryption.StateEncryptionDisabled())
 
 	diag := b.Configure(cty.ObjectVal(map[string]cty.Value{
 		"hostname":     cty.StringVal(tfeHost),
