@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package arguments
@@ -13,6 +15,10 @@ type View struct {
 	// level of noise when multiple instances of the same warning are raised
 	// for a configuration.
 	CompactWarnings bool
+
+	// Concise is used to reduce the level of noise in the output and display
+	// only the important details.
+	Concise bool
 }
 
 // ParseView processes CLI arguments, returning a View value and a
@@ -30,6 +36,8 @@ func ParseView(args []string) (*View, []string) {
 			common.NoColor = true
 		case "-compact-warnings":
 			common.CompactWarnings = true
+		case "-concise":
+			common.Concise = true
 		default:
 			// Unsupported argument: move left to the current position, and
 			// increment the index.
