@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 // Package format contains helpers for formatting various OpenTofu
@@ -32,6 +34,8 @@ func DiffActionSymbol(action plans.Action) string {
 		return "  [yellow]~[reset]"
 	case plans.NoOp:
 		return "   "
+	case plans.Forget:
+		return "  [red].[reset]"
 	default:
 		return "  ?"
 	}
