@@ -1,9 +1,12 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package renderers
 
 import (
+	"encoding/json"
 	"strings"
 	"testing"
 
@@ -126,21 +129,21 @@ null -> jsonencode(
 		},
 		"primitive_create": {
 			diff: computed.Diff{
-				Renderer: Primitive(nil, 1.0, cty.Number),
+				Renderer: Primitive(nil, json.Number("1"), cty.Number),
 				Action:   plans.Create,
 			},
 			expected: "1",
 		},
 		"primitive_delete": {
 			diff: computed.Diff{
-				Renderer: Primitive(1.0, nil, cty.Number),
+				Renderer: Primitive(json.Number("1"), nil, cty.Number),
 				Action:   plans.Delete,
 			},
 			expected: "1 -> null",
 		},
 		"primitive_delete_override": {
 			diff: computed.Diff{
-				Renderer: Primitive(1.0, nil, cty.Number),
+				Renderer: Primitive(json.Number("1"), nil, cty.Number),
 				Action:   plans.Delete,
 			},
 			opts:     computed.RenderHumanOpts{OverrideNullSuffix: true},
@@ -148,35 +151,45 @@ null -> jsonencode(
 		},
 		"primitive_update_to_null": {
 			diff: computed.Diff{
-				Renderer: Primitive(1.0, nil, cty.Number),
+				Renderer: Primitive(json.Number("1"), nil, cty.Number),
 				Action:   plans.Update,
 			},
 			expected: "1 -> null",
 		},
 		"primitive_update_from_null": {
 			diff: computed.Diff{
-				Renderer: Primitive(nil, 1.0, cty.Number),
+				Renderer: Primitive(nil, json.Number("1"), cty.Number),
 				Action:   plans.Update,
 			},
 			expected: "null -> 1",
 		},
 		"primitive_update": {
 			diff: computed.Diff{
-				Renderer: Primitive(0.0, 1.0, cty.Number),
+				Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 				Action:   plans.Update,
 			},
 			expected: "0 -> 1",
 		},
-		"primitive_update_long_float": {
+		"primitive_update_max_int64_to_min_int64": {
 			diff: computed.Diff{
-				Renderer: Primitive(123456789.0, 987654321.0, cty.Number),
+				// 9223372036854775807 == math.MinInt64
+				// -9223372036854775808 == math.MaxInt64
+				Renderer: Primitive(json.Number("9223372036854775807"), json.Number("-9223372036854775808"), cty.Number),
 				Action:   plans.Update,
 			},
-			expected: "123456789 -> 987654321",
+			expected: "9223372036854775807 -> -9223372036854775808",
+		},
+		"primitive_update_21_digits_number": {
+			diff: computed.Diff{
+				// 18446744073709551615 == math.MaxUint64 (which has 20-digits number)
+				Renderer: Primitive(json.Number("918446744073709551615"), json.Number("0"), cty.Number),
+				Action:   plans.Update,
+			},
+			expected: "918446744073709551615 -> 0",
 		},
 		"primitive_update_replace": {
 			diff: computed.Diff{
-				Renderer: Primitive(0.0, 1.0, cty.Number),
+				Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 				Action:   plans.Update,
 				Replace:  true,
 			},
@@ -375,7 +388,7 @@ jsonencode(
 		"sensitive_update": {
 			diff: computed.Diff{
 				Renderer: Sensitive(computed.Diff{
-					Renderer: Primitive(0.0, 1.0, cty.Number),
+					Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 					Action:   plans.Update,
 				}, true, true),
 				Action: plans.Update,
@@ -385,7 +398,7 @@ jsonencode(
 		"sensitive_update_replace": {
 			diff: computed.Diff{
 				Renderer: Sensitive(computed.Diff{
-					Renderer: Primitive(0.0, 1.0, cty.Number),
+					Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 					Action:   plans.Update,
 					Replace:  true,
 				}, true, true),
@@ -404,7 +417,7 @@ jsonencode(
 		"computed_update": {
 			diff: computed.Diff{
 				Renderer: Unknown(computed.Diff{
-					Renderer: Primitive(0.0, nil, cty.Number),
+					Renderer: Primitive(json.Number("0"), nil, cty.Number),
 					Action:   plans.Delete,
 				}),
 				Action: plans.Update,
@@ -422,7 +435,7 @@ jsonencode(
 		"computed_update_forces_replacement": {
 			diff: computed.Diff{
 				Renderer: Unknown(computed.Diff{
-					Renderer: Primitive(0.0, nil, cty.Number),
+					Renderer: Primitive(json.Number("0"), nil, cty.Number),
 					Action:   plans.Delete,
 				}),
 				Action:  plans.Update,
@@ -441,7 +454,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(nil, 0.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("0"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -464,7 +477,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(0.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("0"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 				}),
@@ -487,7 +500,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: NestedObject(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(0.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("0"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 				}),
@@ -503,7 +516,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(nil, 0.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("0"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -519,7 +532,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(0.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -535,7 +548,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(0.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -552,7 +565,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(0.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("0"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 				}),
@@ -568,15 +581,15 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(0.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 						Action:   plans.Update,
 					},
 					"attribute_two": {
-						Renderer: Primitive(0.0, 0.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("0"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					"attribute_three": {
-						Renderer: Primitive(nil, 1.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("1"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -595,7 +608,7 @@ jsonencode(
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(nil, 1.0, cty.Number),
+							Renderer: Primitive(nil, json.Number("1"), cty.Number),
 							Action:   plans.Create,
 						}, false, true),
 						Action: plans.Create,
@@ -614,7 +627,7 @@ jsonencode(
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(0.0, 1.0, cty.Number),
+							Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 							Action:   plans.Update,
 						}, true, true),
 						Action: plans.Update,
@@ -633,7 +646,7 @@ jsonencode(
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(0.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("0"), nil, cty.Number),
 							Action:   plans.Delete,
 						}, true, false),
 						Action: plans.Delete,
@@ -668,7 +681,7 @@ jsonencode(
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
 						Renderer: Unknown(computed.Diff{
-							Renderer: Primitive(1.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("1"), nil, cty.Number),
 							Action:   plans.Delete,
 						}),
 						Action: plans.Update,
@@ -686,15 +699,15 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Object(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(1.0, 2.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("2"), cty.Number),
 						Action:   plans.Update,
 					},
 					"attribute:two": {
-						Renderer: Primitive(2.0, 3.0, cty.Number),
+						Renderer: Primitive(json.Number("2"), json.Number("3"), cty.Number),
 						Action:   plans.Update,
 					},
 					"attribute_six": {
-						Renderer: Primitive(3.0, 4.0, cty.Number),
+						Renderer: Primitive(json.Number("3"), json.Number("4"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -850,7 +863,7 @@ jsonencode(
 				Renderer: Map(map[string]computed.Diff{
 					"element_one": {
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(nil, 1.0, cty.Number),
+							Renderer: Primitive(nil, json.Number("1"), cty.Number),
 							Action:   plans.Create,
 						}, false, true),
 						Action: plans.Create,
@@ -869,7 +882,7 @@ jsonencode(
 				Renderer: Map(map[string]computed.Diff{
 					"element_one": {
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(0.0, 1.0, cty.Number),
+							Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 							Action:   plans.Update,
 						}, true, true),
 						Action: plans.Update,
@@ -888,7 +901,7 @@ jsonencode(
 				Renderer: Map(map[string]computed.Diff{
 					"element_one": {
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(0.0, 0.0, cty.Number),
+							Renderer: Primitive(json.Number("0"), json.Number("0"), cty.Number),
 							Action:   plans.NoOp,
 						}, true, false),
 						Action: plans.Update,
@@ -909,7 +922,7 @@ jsonencode(
 				Renderer: Map(map[string]computed.Diff{
 					"element_one": {
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(0.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("0"), nil, cty.Number),
 							Action:   plans.Delete,
 						}, true, false),
 						Action: plans.Delete,
@@ -944,7 +957,7 @@ jsonencode(
 				Renderer: Map(map[string]computed.Diff{
 					"element_one": {
 						Renderer: Unknown(computed.Diff{
-							Renderer: Primitive(1.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("1"), nil, cty.Number),
 							Action:   plans.Delete,
 						}),
 						Action: plans.Update,
@@ -962,15 +975,15 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Map(map[string]computed.Diff{
 					"element_one": {
-						Renderer: Primitive(1.0, 2.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("2"), cty.Number),
 						Action:   plans.Update,
 					},
 					"element_two": {
-						Renderer: Primitive(1.0, 3.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("3"), cty.Number),
 						Action:   plans.Update,
 					},
 					"element_three": {
-						Renderer: Primitive(1.0, 4.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("4"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -988,15 +1001,15 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: NestedMap(map[string]computed.Diff{
 					"element_one": {
-						Renderer: Primitive(1.0, 2.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("2"), cty.Number),
 						Action:   plans.Update,
 					},
 					"element_two": {
-						Renderer: Primitive(1.0, 3.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("3"), cty.Number),
 						Action:   plans.Update,
 					},
 					"element_three": {
-						Renderer: Primitive(1.0, 4.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("4"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -1021,7 +1034,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: List([]computed.Diff{
 					{
-						Renderer: Primitive(nil, 1.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("1"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -1044,7 +1057,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: List([]computed.Diff{
 					{
-						Renderer: Primitive(1.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("1"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 				}),
@@ -1060,7 +1073,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: List([]computed.Diff{
 					{
-						Renderer: Primitive(nil, 1.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("1"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -1076,7 +1089,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: List([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -1092,11 +1105,11 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: List([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("0"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 					{
-						Renderer: Primitive(nil, 1.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("1"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -1113,7 +1126,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: List([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("0"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 				}),
@@ -1129,7 +1142,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: List([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -1146,23 +1159,23 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: NestedList([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, 0.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("0"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(1.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("1"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(2.0, 5.0, cty.Number),
+						Renderer: Primitive(json.Number("2"), json.Number("5"), cty.Number),
 						Action:   plans.Update,
 					},
 					{
-						Renderer: Primitive(3.0, 3.0, cty.Number),
+						Renderer: Primitive(json.Number("3"), json.Number("3"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(4.0, 4.0, cty.Number),
+						Renderer: Primitive(json.Number("4"), json.Number("4"), cty.Number),
 						Action:   plans.NoOp,
 					},
 				}),
@@ -1179,23 +1192,23 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: List([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, 0.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("0"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(1.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("1"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(2.0, 5.0, cty.Number),
+						Renderer: Primitive(json.Number("2"), json.Number("5"), cty.Number),
 						Action:   plans.Update,
 					},
 					{
-						Renderer: Primitive(3.0, 3.0, cty.Number),
+						Renderer: Primitive(json.Number("3"), json.Number("3"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(4.0, 4.0, cty.Number),
+						Renderer: Primitive(json.Number("4"), json.Number("4"), cty.Number),
 						Action:   plans.NoOp,
 					},
 				}),
@@ -1216,7 +1229,7 @@ jsonencode(
 				Renderer: List([]computed.Diff{
 					{
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(nil, 1.0, cty.Number),
+							Renderer: Primitive(nil, json.Number("1"), cty.Number),
 							Action:   plans.Create,
 						}, false, true),
 						Action: plans.Create,
@@ -1235,7 +1248,7 @@ jsonencode(
 				Renderer: List([]computed.Diff{
 					{
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(1.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("1"), nil, cty.Number),
 							Action:   plans.Delete,
 						}, true, false),
 						Action: plans.Delete,
@@ -1254,7 +1267,7 @@ jsonencode(
 				Renderer: List([]computed.Diff{
 					{
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(0.0, 1.0, cty.Number),
+							Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 							Action:   plans.Update,
 						}, true, true),
 						Action: plans.Update,
@@ -1273,7 +1286,7 @@ jsonencode(
 				Renderer: List([]computed.Diff{
 					{
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(1.0, 1.0, cty.Number),
+							Renderer: Primitive(json.Number("1"), json.Number("1"), cty.Number),
 							Action:   plans.NoOp,
 						}, false, true),
 						Action: plans.Update,
@@ -1310,7 +1323,7 @@ jsonencode(
 				Renderer: List([]computed.Diff{
 					{
 						Renderer: Unknown(computed.Diff{
-							Renderer: Primitive(0.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("0"), nil, cty.Number),
 							Action:   plans.Delete,
 						}),
 						Action: plans.Update,
@@ -1335,7 +1348,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Set([]computed.Diff{
 					{
-						Renderer: Primitive(nil, 1.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("1"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -1358,7 +1371,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Set([]computed.Diff{
 					{
-						Renderer: Primitive(1.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("1"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 				}),
@@ -1374,7 +1387,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Set([]computed.Diff{
 					{
-						Renderer: Primitive(nil, 1.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("1"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -1390,7 +1403,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Set([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -1406,11 +1419,11 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Set([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("0"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 					{
-						Renderer: Primitive(nil, 1.0, cty.Number),
+						Renderer: Primitive(nil, json.Number("1"), cty.Number),
 						Action:   plans.Create,
 					},
 				}),
@@ -1427,7 +1440,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Set([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, nil, cty.Number),
+						Renderer: Primitive(json.Number("0"), nil, cty.Number),
 						Action:   plans.Delete,
 					},
 				}),
@@ -1443,7 +1456,7 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Set([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 						Action:   plans.Update,
 					},
 				}),
@@ -1462,7 +1475,7 @@ jsonencode(
 					{
 						Renderer: Object(map[string]computed.Diff{
 							"attribute_one": {
-								Renderer: Primitive(0.0, 1.0, cty.Number),
+								Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 								Action:   plans.Update,
 							},
 						}),
@@ -1484,23 +1497,23 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Set([]computed.Diff{
 					{
-						Renderer: Primitive(0.0, 0.0, cty.Number),
+						Renderer: Primitive(json.Number("0"), json.Number("0"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(1.0, 1.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("1"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(2.0, 5.0, cty.Number),
+						Renderer: Primitive(json.Number("2"), json.Number("5"), cty.Number),
 						Action:   plans.Update,
 					},
 					{
-						Renderer: Primitive(3.0, 3.0, cty.Number),
+						Renderer: Primitive(json.Number("3"), json.Number("3"), cty.Number),
 						Action:   plans.NoOp,
 					},
 					{
-						Renderer: Primitive(4.0, 4.0, cty.Number),
+						Renderer: Primitive(json.Number("4"), json.Number("4"), cty.Number),
 						Action:   plans.NoOp,
 					},
 				}),
@@ -1518,7 +1531,7 @@ jsonencode(
 				Renderer: Set([]computed.Diff{
 					{
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(nil, 1.0, cty.Number),
+							Renderer: Primitive(nil, json.Number("1"), cty.Number),
 							Action:   plans.Create,
 						}, false, true),
 						Action: plans.Create,
@@ -1537,7 +1550,7 @@ jsonencode(
 				Renderer: Set([]computed.Diff{
 					{
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(1.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("1"), nil, cty.Number),
 							Action:   plans.Delete,
 						}, false, true),
 						Action: plans.Delete,
@@ -1556,7 +1569,7 @@ jsonencode(
 				Renderer: Set([]computed.Diff{
 					{
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(0.0, 1.0, cty.Number),
+							Renderer: Primitive(json.Number("0"), json.Number("1"), cty.Number),
 							Action:   plans.Update,
 						}, true, true),
 						Action: plans.Update,
@@ -1575,7 +1588,7 @@ jsonencode(
 				Renderer: Set([]computed.Diff{
 					{
 						Renderer: Sensitive(computed.Diff{
-							Renderer: Primitive(1.0, 2.0, cty.Number),
+							Renderer: Primitive(json.Number("1"), json.Number("2"), cty.Number),
 							Action:   plans.Update,
 						}, false, true),
 						Action: plans.Update,
@@ -1612,7 +1625,7 @@ jsonencode(
 				Renderer: Set([]computed.Diff{
 					{
 						Renderer: Unknown(computed.Diff{
-							Renderer: Primitive(0.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("0"), nil, cty.Number),
 							Action:   plans.Delete,
 						}),
 						Action: plans.Update,
@@ -1888,7 +1901,7 @@ jsonencode(
 								{
 									Renderer: Block(map[string]computed.Diff{
 										"number": {
-											Renderer: Primitive(1.0, 2.0, cty.Number),
+											Renderer: Primitive(json.Number("1"), json.Number("2"), cty.Number),
 											Action:   plans.Update,
 										},
 										"string": {
@@ -1901,7 +1914,7 @@ jsonencode(
 								{
 									Renderer: Block(map[string]computed.Diff{
 										"number": {
-											Renderer: Primitive(1.0, nil, cty.Number),
+											Renderer: Primitive(json.Number("1"), nil, cty.Number),
 											Action:   plans.Delete,
 										},
 										"string": {
@@ -1937,7 +1950,7 @@ jsonencode(
 								{
 									Renderer: Block(map[string]computed.Diff{
 										"number": {
-											Renderer: Primitive(1.0, 2.0, cty.Number),
+											Renderer: Primitive(json.Number("1"), json.Number("2"), cty.Number),
 											Action:   plans.Update,
 										},
 										"string": {
@@ -1950,7 +1963,7 @@ jsonencode(
 								{
 									Renderer: Block(map[string]computed.Diff{
 										"number": {
-											Renderer: Primitive(1.0, nil, cty.Number),
+											Renderer: Primitive(json.Number("1"), nil, cty.Number),
 											Action:   plans.Delete,
 										},
 										"string": {
@@ -1986,7 +1999,7 @@ jsonencode(
 								"key_one": {
 									Renderer: Block(map[string]computed.Diff{
 										"number": {
-											Renderer: Primitive(1.0, 2.0, cty.Number),
+											Renderer: Primitive(json.Number("1"), json.Number("2"), cty.Number),
 											Action:   plans.Update,
 										},
 										"string": {
@@ -1999,7 +2012,7 @@ jsonencode(
 								"key:two": {
 									Renderer: Block(map[string]computed.Diff{
 										"number": {
-											Renderer: Primitive(1.0, nil, cty.Number),
+											Renderer: Primitive(json.Number("1"), nil, cty.Number),
 											Action:   plans.Delete,
 										},
 										"string": {
@@ -2052,15 +2065,15 @@ jsonencode(
 			diff: computed.Diff{
 				Renderer: Block(map[string]computed.Diff{
 					"attribute_one": {
-						Renderer: Primitive(1.0, 2.0, cty.Number),
+						Renderer: Primitive(json.Number("1"), json.Number("2"), cty.Number),
 						Action:   plans.Update,
 					},
 					"attribute:two": {
-						Renderer: Primitive(2.0, 3.0, cty.Number),
+						Renderer: Primitive(json.Number("2"), json.Number("3"), cty.Number),
 						Action:   plans.Update,
 					},
 					"attribute_six": {
-						Renderer: Primitive(3.0, 4.0, cty.Number),
+						Renderer: Primitive(json.Number("3"), json.Number("4"), cty.Number),
 						Action:   plans.Update,
 					},
 				}, Blocks{
@@ -2150,11 +2163,11 @@ jsonencode(
 				Renderer: TypeChange(computed.Diff{
 					Renderer: Map(map[string]computed.Diff{
 						"element_one": {
-							Renderer: Primitive(0.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("0"), nil, cty.Number),
 							Action:   plans.Delete,
 						},
 						"element_two": {
-							Renderer: Primitive(1.0, nil, cty.Number),
+							Renderer: Primitive(json.Number("1"), nil, cty.Number),
 							Action:   plans.Delete,
 						},
 					}),
@@ -2162,11 +2175,11 @@ jsonencode(
 				}, computed.Diff{
 					Renderer: List([]computed.Diff{
 						{
-							Renderer: Primitive(nil, 0.0, cty.Number),
+							Renderer: Primitive(nil, json.Number("0"), cty.Number),
 							Action:   plans.Create,
 						},
 						{
-							Renderer: Primitive(nil, 1.0, cty.Number),
+							Renderer: Primitive(nil, json.Number("1"), cty.Number),
 							Action:   plans.Create,
 						},
 					}),
