@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package configs
@@ -254,13 +256,9 @@ func validateProviderConfigsForTests(cfg *Config) (diags hcl.Diagnostics) {
 				// Let's make a little fake module call that we can use to call
 				// into validateProviderConfigs.
 				mc := &ModuleCall{
-					Name:            run.Name,
-					SourceAddr:      run.Module.Source,
-					SourceAddrRange: run.Module.SourceDeclRange,
-					SourceSet:       true,
-					Version:         run.Module.Version,
-					Providers:       providers,
-					DeclRange:       run.Module.DeclRange,
+					Name:      run.Name,
+					Providers: providers,
+					DeclRange: run.Module.DeclRange,
 				}
 
 				diags = append(diags, validateProviderConfigs(mc, run.ConfigUnderTest, nil)...)
