@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package configs
@@ -10,8 +12,10 @@ import (
 	"github.com/hashicorp/hcl/v2/gohcl"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
 	hcljson "github.com/hashicorp/hcl/v2/json"
+	"github.com/zclconf/go-cty/cty"
 
 	"github.com/opentofu/opentofu/internal/addrs"
+	"github.com/opentofu/opentofu/internal/configs/hcl2shim"
 	"github.com/opentofu/opentofu/internal/lang"
 	"github.com/opentofu/opentofu/internal/tfdiags"
 )
@@ -47,6 +51,14 @@ type Resource struct {
 	// If this is nil, then this resource is essentially public.
 	Container Container
 
+	// IsOverridden indicates if the resource is being overridden. It's used in
+	// testing framework to not call the underlying provider.
+	IsOverridden bool
+	// OverrideValues are only valid if IsOverridden is set to true. The values
+	// should be used to compose mock provider response. It is possible to have
+	// zero-length OverrideValues even if IsOverridden is set to true.
+	OverrideValues map[string]cty.Value
+
 	DeclRange hcl.Range
 	TypeRange hcl.Range
 }
@@ -549,18 +561,9 @@ func decodeReplaceTriggeredBy(expr hcl.Expression) ([]hcl.Expression, hcl.Diagno
 
 	for i, expr := range exprs {
 		if isJSON {
-			// We can abuse the hcl json api and rely on the fact that calling
-			// Value on a json expression with no EvalContext will return the
-			// raw string. We can then parse that as normal hcl syntax, and
-			// continue with the decoding.
-			v, ds := expr.Value(nil)
-			diags = diags.Extend(ds)
-			if diags.HasErrors() {
-				continue
-			}
-
-			expr, ds = hclsyntax.ParseExpression([]byte(v.AsString()), "", expr.Range().Start)
-			diags = diags.Extend(ds)
+			var convertDiags hcl.Diagnostics
+			expr, convertDiags = hcl2shim.ConvertJSONExpressionToHCL(expr)
+			diags = diags.Extend(convertDiags)
 			if diags.HasErrors() {
 				continue
 			}
