@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package providers
@@ -14,6 +16,11 @@ import (
 // Interface represents the set of methods required for a complete resource
 // provider plugin.
 type Interface interface {
+	// GetMetadata is not yet implemented or used at this time.  It may
+	// be used in the future to avoid loading a provider's full schema
+	// for initial validation.  This could result in some potential
+	// memory savings.
+
 	// GetSchema returns the complete schema for the provider.
 	GetProviderSchema() GetProviderSchemaResponse
 
@@ -70,6 +77,13 @@ type Interface interface {
 	// ReadDataSource returns the data source's current state.
 	ReadDataSource(ReadDataSourceRequest) ReadDataSourceResponse
 
+	// GetFunctions returns a full list of functions defined in this provider.  It should be a super
+	// set of the functions returned in GetProviderSchema()
+	GetFunctions() GetFunctionsResponse
+
+	// CallFunction requests that the given function is called and response returned.
+	CallFunction(CallFunctionRequest) CallFunctionResponse
+
 	// Close shuts down the plugin process if applicable.
 	Close() error
 }
@@ -97,6 +111,9 @@ type GetProviderSchemaResponse struct {
 
 	// ServerCapabilities lists optional features supported by the provider.
 	ServerCapabilities ServerCapabilities
+
+	// Functions lists all functions supported by this provider.
+	Functions map[string]FunctionSpec
 }
 
 // Schema pairs a provider or resource schema with that schema's version.
@@ -128,6 +145,47 @@ type ServerCapabilities struct {
 	GetProviderSchemaOptional bool
 }
 
+type FunctionSpec struct {
+	// List of parameters required to call the function
+	Parameters []FunctionParameterSpec
+	// Optional Spec for variadic parameters
+	VariadicParameter *FunctionParameterSpec
+	// Type which the function will return
+	Return cty.Type
+	// Human-readable shortened documentation for the function
+	Summary string
+	// Human-readable documentation for the function
+	Description string
+	// Formatting type of the Description field
+	DescriptionFormat TextFormatting
+	// Human-readable message present if the function is deprecated
+	DeprecationMessage string
+}
+
+type FunctionParameterSpec struct {
+	// Human-readable display name for the parameter
+	Name string
+	// Type constraint for the parameter
+	Type cty.Type
+	// Null values alowed for the parameter
+	AllowNullValue bool
+	// Unknown Values allowed for the parameter
+	// Individual provider implementations may interpret this as a
+	// check using IsWhollyKnown instead cty's default of IsKnown.
+	// If the input is not wholly known, the result should be
+	// cty.UnknownVal(spec.returnType)
+	AllowUnknownValues bool
+	// Human-readable documentation for the parameter
+	Description string
+	// Formatting type of the Description field
+	DescriptionFormat TextFormatting
+}
+
+type TextFormatting string
+
+const TextFormattingPlain = TextFormatting("Plain")
+const TextFormattingMarkdown = TextFormatting("Markdown")
+
 type ValidateProviderConfigRequest struct {
 	// Config is the raw configuration value for the provider.
 	Config cty.Value
@@ -419,3 +477,28 @@ type ReadDataSourceResponse struct {
 	// Diagnostics contains any warnings or errors from the method call.
 	Diagnostics tfdiags.Diagnostics
 }
+
+type GetFunctionsResponse struct {
+	Functions map[string]FunctionSpec
+
+	Diagnostics tfdiags.Diagnostics
+}
+
+type CallFunctionRequest struct {
+	Name      string
+	Arguments []cty.Value
+}
+
+type CallFunctionResponse struct {
+	Result cty.Value
+	Error  error
+}
+
+type CallFunctionArgumentError struct {
+	Text             string
+	FunctionArgument int
+}
+
+func (err *CallFunctionArgumentError) Error() string {
+	return err.Text
+}
