@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package tofu
@@ -39,6 +41,7 @@ type Evaluator struct {
 	// Config is the root node in the configuration tree.
 	Config *configs.Config
 
+	VariableValuesLock *sync.Mutex
 	// VariableValues is a map from variable names to their associated values,
 	// within the module indicated by ModulePath. VariableValues is modified
 	// concurrently, and so it must be accessed only while holding
@@ -46,8 +49,7 @@ type Evaluator struct {
 	//
 	// The first map level is string representations of addr.ModuleInstance
 	// values, while the second level is variable names.
-	VariableValues     map[string]map[string]cty.Value
-	VariableValuesLock *sync.Mutex
+	VariableValues map[string]map[string]cty.Value
 
 	// Plugins is the library of available plugin components (providers and
 	// provisioners) that we have available to help us evaluate expressions
@@ -74,15 +76,16 @@ type Evaluator struct {
 // If the "self" argument is nil then the "self" object is not available
 // in evaluated expressions. Otherwise, it behaves as an alias for the given
 // address.
-func (e *Evaluator) Scope(data lang.Data, self addrs.Referenceable, source addrs.Referenceable) *lang.Scope {
+func (e *Evaluator) Scope(data lang.Data, self addrs.Referenceable, source addrs.Referenceable, functions lang.ProviderFunction) *lang.Scope {
 	return &lang.Scope{
-		Data:          data,
-		ParseRef:      addrs.ParseRef,
-		SelfAddr:      self,
-		SourceAddr:    source,
-		PureOnly:      e.Operation != walkApply && e.Operation != walkDestroy && e.Operation != walkEval,
-		BaseDir:       ".", // Always current working directory for now.
-		PlanTimestamp: e.PlanTimestamp,
+		Data:              data,
+		ParseRef:          addrs.ParseRef,
+		SelfAddr:          self,
+		SourceAddr:        source,
+		PureOnly:          e.Operation != walkApply && e.Operation != walkDestroy && e.Operation != walkEval,
+		BaseDir:           ".", // Always current working directory for now.
+		PlanTimestamp:     e.PlanTimestamp,
+		ProviderFunctions: functions,
 	}
 }
 
@@ -739,8 +742,8 @@ func (d *evaluationStateData) GetResource(addr addrs.Resource, rng tfdiags.Sourc
 	// Decode all instances in the current state
 	instances := map[addrs.InstanceKey]cty.Value{}
 	pendingDestroy := d.Operation == walkDestroy
-	for key, is := range rs.Instances {
-		if is == nil || is.Current == nil {
+	for key, instance := range rs.Instances {
+		if instance == nil || instance.Current == nil {
 			// Assume we're dealing with an instance that hasn't been created yet.
 			instances[key] = cty.UnknownVal(ty)
 			continue
@@ -763,7 +766,7 @@ func (d *evaluationStateData) GetResource(addr addrs.Resource, rng tfdiags.Sourc
 
 		// Planned resources are temporarily stored in state with empty values,
 		// and need to be replaced by the planned value here.
-		if is.Current.Status == states.ObjectPlanned {
+		if instance.Current.Status == states.ObjectPlanned {
 			if change == nil {
 				// If the object is in planned status then we should not get
 				// here, since we should have found a pending value in the plan
@@ -787,17 +790,20 @@ func (d *evaluationStateData) GetResource(addr addrs.Resource, rng tfdiags.Sourc
 				continue
 			}
 
-			// If our provider schema contains sensitive values, mark those as sensitive
 			afterMarks := change.AfterValMarks
 			if schema.ContainsSensitive() {
-				afterMarks = append(afterMarks, schema.ValueMarks(val, nil)...)
+				// Now that we know that the schema contains sensitive marks,
+				// Combine those marks together to ensure that the value is marked correctly but not double marked
+				schemaMarks := schema.ValueMarks(val, nil)
+				afterMarks = combinePathValueMarks(afterMarks, schemaMarks)
 			}
 
 			instances[key] = val.MarkWithPaths(afterMarks)
+
 			continue
 		}
 
-		ios, err := is.Current.Decode(ty)
+		instanceObjectSrc, err := instance.Current.Decode(ty)
 		if err != nil {
 			// This shouldn't happen, since by the time we get here we
 			// should have upgraded the state data already.
@@ -810,17 +816,17 @@ func (d *evaluationStateData) GetResource(addr addrs.Resource, rng tfdiags.Sourc
 			continue
 		}
 
-		val := ios.Value
+		val := instanceObjectSrc.Value
 
-		// If our schema contains sensitive values, mark those as sensitive.
-		// Since decoding the instance object can also apply sensitivity marks,
-		// we must remove and combine those before remarking to avoid a double-
-		// mark error.
 		if schema.ContainsSensitive() {
 			var marks []cty.PathValueMarks
+			// Now that we know that the schema contains sensitive marks,
+			// Combine those marks together to ensure that the value is marked correctly but not double marked
 			val, marks = val.UnmarkDeepWithPaths()
-			marks = append(marks, schema.ValueMarks(val, nil)...)
-			val = val.MarkWithPaths(marks)
+			schemaMarks := schema.ValueMarks(val, nil)
+
+			combined := combinePathValueMarks(marks, schemaMarks)
+			val = val.MarkWithPaths(combined)
 		}
 		instances[key] = val
 	}
