@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package tofu
@@ -50,7 +52,23 @@ type NodePlannableResourceInstance struct {
 
 	// importTarget, if populated, contains the information necessary to plan
 	// an import of this resource.
-	importTarget ImportTarget
+	importTarget EvaluatedConfigImportTarget
+}
+
+// EvaluatedConfigImportTarget is a target that we need to import. It's created when an import target originated from
+// an import block, after everything regarding the configuration has been evaluated.
+// At this point, the import target is of a single resource instance
+type EvaluatedConfigImportTarget struct {
+	// Config is the original import block for this import. This might be null
+	// if the import did not originate in config.
+	Config *configs.Import
+
+	// Addr is the actual address of the resource instance that we should import into. At this point, the address
+	// should be fully evaluated
+	Addr addrs.AbsResourceInstance
+
+	// ID is the string ID of the resource to import. This is resource-instance specific.
+	ID string
 }
 
 var (
@@ -159,18 +177,7 @@ func (n *NodePlannableResourceInstance) managedResourceExecute(ctx EvalContext)
 		}
 	}
 
-	importing := n.importTarget.ID != nil
-	var importId string
-
-	if importing {
-		var evalDiags tfdiags.Diagnostics
-
-		importId, evalDiags = evaluateImportIdExpression(n.importTarget.ID, ctx)
-		if evalDiags.HasErrors() {
-			diags = diags.Append(evalDiags)
-			return diags
-		}
-	}
+	importing := n.shouldImport(ctx)
 
 	if importing && n.Config == nil && len(n.generateConfigPath) == 0 {
 		// Then the user wrote an import target to a target that didn't exist.
@@ -185,12 +192,7 @@ func (n *NodePlannableResourceInstance) managedResourceExecute(ctx EvalContext)
 			// You can't generate config for a resource that is inside a
 			// module, so we will present a different error message for
 			// this case.
-			diags = diags.Append(&hcl.Diagnostic{
-				Severity: hcl.DiagError,
-				Summary:  "Import block target does not exist",
-				Detail:   "The target for the given import block does not exist. The specified target is within a module, and must be defined as a resource within that module before anything can be imported.",
-				Subject:  n.importTarget.Config.DeclRange.Ptr(),
-			})
+			diags = diags.Append(importResourceWithoutConfigDiags(n.Addr.String(), n.importTarget.Config))
 		}
 		return diags
 	}
@@ -198,7 +200,7 @@ func (n *NodePlannableResourceInstance) managedResourceExecute(ctx EvalContext)
 	// If the resource is to be imported, we now ask the provider for an Import
 	// and a Refresh, and save the resulting state to instanceRefreshState.
 	if importing {
-		instanceRefreshState, diags = n.importState(ctx, addr, importId, provider, providerSchema)
+		instanceRefreshState, diags = n.importState(ctx, addr, n.importTarget.ID, provider, providerSchema)
 	} else {
 		var readDiags tfdiags.Diagnostics
 		instanceRefreshState, readDiags = n.readResourceInstanceState(ctx, addr)
@@ -308,7 +310,7 @@ func (n *NodePlannableResourceInstance) managedResourceExecute(ctx EvalContext)
 		}
 
 		if importing {
-			change.Importing = &plans.Importing{ID: importId}
+			change.Importing = &plans.Importing{ID: n.importTarget.ID}
 		}
 
 		// FIXME: here we udpate the change to reflect the reason for
@@ -318,11 +320,6 @@ func (n *NodePlannableResourceInstance) managedResourceExecute(ctx EvalContext)
 			change.ActionReason = plans.ResourceInstanceReplaceByTriggers
 		}
 
-		diags = diags.Append(n.checkPreventDestroy(change))
-		if diags.HasErrors() {
-			return diags
-		}
-
 		// FIXME: it is currently important that we write resource changes to
 		// the plan (n.writeChange) before we write the corresponding state
 		// (n.writeResourceInstanceState).
@@ -338,6 +335,13 @@ func (n *NodePlannableResourceInstance) managedResourceExecute(ctx EvalContext)
 		// update these two data structures incorrectly through any objects
 		// reachable via the tofu.EvalContext API.
 		diags = diags.Append(n.writeChange(ctx, change, ""))
+		if diags.HasErrors() {
+			return diags
+		}
+		diags = diags.Append(n.checkPreventDestroy(change))
+		if diags.HasErrors() {
+			return diags
+		}
 
 		diags = diags.Append(n.writeResourceInstanceState(ctx, instancePlanState, workingState))
 		if diags.HasErrors() {
@@ -505,7 +509,7 @@ func (n *NodePlannableResourceInstance) importState(ctx EvalContext, addr addrs.
 	}))
 
 	if imported[0].TypeName == "" {
-		diags = diags.Append(fmt.Errorf("import of %s didn't set type", n.importTarget.Addr.String()))
+		diags = diags.Append(fmt.Errorf("import of %s didn't set type", n.Addr.String()))
 		return nil, diags
 	}
 
@@ -524,7 +528,7 @@ func (n *NodePlannableResourceInstance) importState(ctx EvalContext, addr addrs.
 
 	// refresh
 	riNode := &NodeAbstractResourceInstance{
-		Addr: n.importTarget.Addr,
+		Addr: n.Addr,
 		NodeAbstractResource: NodeAbstractResource{
 			ResolvedProvider: n.ResolvedProvider,
 		},
@@ -548,12 +552,41 @@ func (n *NodePlannableResourceInstance) importState(ctx EvalContext, addr addrs.
 					"is correct and that it is associated with the provider's "+
 					"configured region or endpoint, or use \"tofu apply\" to "+
 					"create a new remote object for this resource.",
-				n.importTarget.Addr,
+				n.Addr,
 			),
 		))
 		return instanceRefreshState, diags
 	}
 
+	// Insert marks from configuration
+	if n.Config != nil {
+		keyData := EvalDataForNoInstanceKey
+
+		switch {
+		case n.Config.Count != nil:
+			keyData = InstanceKeyEvalData{
+				CountIndex: cty.UnknownVal(cty.Number),
+			}
+		case n.Config.ForEach != nil:
+			keyData = InstanceKeyEvalData{
+				EachKey:   cty.UnknownVal(cty.String),
+				EachValue: cty.UnknownVal(cty.DynamicPseudoType),
+			}
+		}
+
+		valueWithConfigurationSchemaMarks, _, configDiags := ctx.EvaluateBlock(n.Config.Config, n.Schema, nil, keyData)
+		diags = diags.Append(configDiags)
+		if configDiags.HasErrors() {
+			return instanceRefreshState, diags
+		}
+
+		_, marks := instanceRefreshState.Value.UnmarkDeepWithPaths()
+		_, configSchemaMarks := valueWithConfigurationSchemaMarks.UnmarkDeepWithPaths()
+		merged := combinePathValueMarks(marks, configSchemaMarks)
+
+		instanceRefreshState.Value = instanceRefreshState.Value.MarkWithPaths(merged)
+	}
+
 	// If we're importing and generating config, generate it now.
 	if len(n.generateConfigPath) > 0 {
 		if n.Config != nil {
@@ -607,6 +640,17 @@ func (n *NodePlannableResourceInstance) importState(ctx EvalContext, addr addrs.
 	return instanceRefreshState, diags
 }
 
+func (n *NodePlannableResourceInstance) shouldImport(ctx EvalContext) bool {
+	if n.importTarget.ID == "" {
+		return false
+	}
+
+	// If the import target already has a state - we should not attempt to import it, but instead run a normal plan
+	// for it
+	state := ctx.State()
+	return state.ResourceInstance(n.ResourceInstanceAddr()) == nil
+}
+
 // generateHCLStringAttributes produces a string in HCL format for the given
 // resource state and schema without the surrounding block.
 func (n *NodePlannableResourceInstance) generateHCLStringAttributes(addr addrs.AbsResourceInstance, state *states.ResourceInstanceObject, schema *configschema.Block) (string, tfdiags.Diagnostics) {
