@@ -1,5 +1,7 @@
 #!/usr/bin/env bash
-# Copyright (c) HashiCorp, Inc.
+# Copyright (c) The OpenTofu Authors
+# SPDX-License-Identifier: MPL-2.0
+# Copyright (c) 2023 HashiCorp, Inc.
 # SPDX-License-Identifier: MPL-2.0
 
 
@@ -9,7 +11,7 @@ echo "==> Checking that code complies with static analysis requirements..."
 skip="internal/legacy|backend/remote-state/"
 
 # Skip generated code for protobufs.
-skip=$skip"|internal/planproto|internal/tfplugin5|internal/tfplugin6"
+skip=$skip"|internal/planproto|internal/tfplugin5|internal/tfplugin6|internal/gohcl"
 
 packages=$(go list ./... | egrep -v ${skip})
 
